//////////////////////////////////////////////////////////////////////
// StirMark Benchmark - SMBReconstructor.h
//
// Contents: Interface for the CSMBReconstructor class
//
// Purpose:  Reconstructors are functions used to interpolate the value of a
//           sampled image at non-integer pixel coordinates. Typical
//           Typical reconstructors include simple linear interpolation,
//           Nyquist interpolation, nearest neighbour or quadratic,
//           interpolation.
//           Convolution filters also fit the same model.
//
//           Note that the coordinates in this sub-system obey the
//           idea set out in:
//             Paul S. Heckbert, What are the coordinates of a pixel,
//             in Andrew S. Glassner Ed., Graphics Gems, pp. 246--248, 1990.
//             ISBN 0-12-286165-5
//           The discrete coordinates are offset from the continuous ones by
//           half a unit:
//
//                 0       1       2       3
//                 *       *       *       *      discrete
//             +-------+-------+-------+-------+  continuous
//             0       1       2       3       4
//
//           Thus: the conversion from continuous to discrete is:
//             discrete = ROUND(continuous)
//           i.e. pixel 0 goes from 0.0 to 0.999...;
//                pixel 1 from 1.0 to 1.999... etc.
//           Thus the continuous coordinate of the centre of pixel
//           n is (n + 0.5)
//
// Created:  Fabien A. P. Petitcolas, Microsoft Research, March 2000
//
// Modified: Nazim Fats, Microsoft Research
//
// History:
//
// Copyright (c) 2000-2002, Microsoft Research Ltd , Institut National
// de Recherche en Informatique et Automatique (INRIA), Institut Eurcom
// and the Integrated Publication and Information Systems Institute at
// GMD - Forschungszentrum Informationstechnik GmbH (GMD-IPSI).
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted for non-commercial research and academic
// use only, provided that the following conditions are met:
// 
// - Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer. Each
//   individual file must retain its own copyright notice.
// 
// - Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions, the following disclaimer and the
//   list of contributors in the documentation and/or other materials
//   provided with the distribution.
// 
// - Modification of the program or portion of it is allowed provided
//   that the modified files carry prominent notices stating where and
//   when they have been changed. If you do modify this program you
//   should send to the contributors a general description of the changes
//   and send them a copy of your changes at their request. By sending
//   any changes to this program to the contributors, you are granting a
//   license on such changes under the same terms and conditions as
//   provided in this license agreement. However, the contributors are
//   under no obligation to accept your changes.
// 
// - All non-commercial advertising materials mentioning features or use
//   of this software must display the following acknowledgement:
// 
//     This product includes software developed by Microsoft Research
//     Ltd, Institut National de Recherche en Informatique et Automatique
//     (INRIA), Institut Eurcom and the Integrated Publication and
//     Information Systems Institute at GMD - Forschungszentrum
//     Informationstechnik GmbH (GMD-IPSI).
// 
// - Neither name of Microsoft Research Ltd, INRIA, Eurcom and GMD-IPSI
//   nor the names of their contributors may be used to endorse or
//   promote products derived from this software without specific prior
//   written permission.
// 
// - If you use StirMark Benchmark for your research, please cite:
// 
//     Fabien A. P. Petitcolas, Martin Steinebach, Frdric Raynal, Jana
//     Dittmann, Caroline Fontaine, Nazim Fats. A public automated
//     web-based evaluation service for watermarking schemes: StirMark
//     Benchmark. In Ping Wah Wong and Edward J. Delp, editors,
//     proceedings of electronic imaging, security and watermarking of
//     multimedia contents III, vol. 4314, San Jose, California, U.S.A.,
//     20-26 January 2001. The Society for imaging science and
//     technology (I.S.&T.) and the international Society for optical
//     engineering (SPIE). ISSN 0277-786X.
// 
// and
// 
//     Fabien A. P. Petitcolas. Watermarking schemes
//     evaluation. I.E.E.E. Signal Processing, vol. 17, no. 5,
//     pp. 58-64, September 2000.
// 
// THIS SOFTWARE IS NOT INTENDED FOR ANY COMMERCIAL APPLICATION AND IS
// PROVIDED BY MICROSOFT RESEARCH LTD, INRIA, EURCOM, GMD-IPSI AND
// CONTRIBUTORS 'AS IS', WITH ALL FAULTS AND ANY EXPRESS OR IMPLIED
// REPRESENTATIONS OR WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE, TITLE OR NONINFRINGEMENT OF INTELLECTUAL
// PROPERTY ARE DISCLAIMED. IN NO EVENT SHALL MICROSOFT RESEARCH LTD,
// INRIA, EURCOM, GMD-IPSI OR THEIR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
// IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// THE USE OF THIS SOFTWARE FOR CIRCUMVENTING WITHOUT AUTHORITY ANY
// EFFECTIVE TECHNOLOGICAL MEASURES DESIGNED TO PROTECT ANY COPYRIGHTS OR
// ANY RIGHTS RELATED TO COPYRIGHT AS PROVIDED BY LAW OR THE SUI GENERIS
// RIGHT PROVIDED BY SOME COUNTRIES IS STRICTLY PROHIBITED.
//
// Part of this implementation (some of the reconstructors) is based
// on code originally written by Neil Dodgson (nad@cl.cam.ac.uk)
// for another project. The resampling algorithms are explained in:
//   Neil Dodgson, "Quadratic Interpolation for Image Resampling,"
//   IEEE Transactions on Image Processing, vol. 6, no. 9,
//   pp. 1322--1326, Sept. 1997.
//
// $Header: /home/cvs/StirmarkBench/StirMark_Bench/GeometricTransforms/SMBReconstructor.h,v 1.11 2002/04/19 10:23:57 petitcolas Exp $
//////////////////////////////////////////////////////////////////////

#if !defined(_SMB_RECONSTRUCTOR_H_)
#define      _SMB_RECONSTRUCTOR_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



// STL headers

// Other standard headers

// SMB headers
#include "../Base/StirmarkBenchmark.h"

// Other definitions
class CSMBImage;



class CSMBReconstructor  
{
public:
  CSMBReconstructor() {};
  virtual ~CSMBReconstructor() {};

  virtual double Reconstruct(double            in_x,
                             double            in_y,
                             uint              in_channel, 
                             const CSMBImage & in_img) const = 0;
};

#endif // !defined(_SMB_RECONSTRUCTOR_H_)
